#!/usr/bin/perl -w

package Data::Array2ArrayMap;
use Carp;
use strict;

use vars qw($VERSION);
$VERSION = '0.12';

=head1 NAME

Data::Array2ArrayMap - perl interface for hierarchical data storage

=head1 SYNOPSIS

This help is only on perl interface of Data::Array2ArrayMap.

First you 

For example, 

=head1 DESCRIPTION

L<Hash::MultiKey>,
L<Tie::ListKeyedHash>,
L<Data::Hive>,
L<DBM::Deep>,

=head1 METHODS

=cut

=head3 new()

new(backend-specific arguments) is generally backend-specific.

=over 4

=item Common Options

=over 4 

=item *

strict - if set to 0 the module will allow 

=back

=back

=cut

sub new {
    my $class  = shift;
    my $self = {};
    bless($self,$class);
    return $self;
}


=head2 Query Capabilities Methods

Those methods allow quering of backend's capabilities.

=head3 is_readonly()

=cut
sub is_readonly() {return 1}

=head3 is_persistent_storage()

=cut
sub is_persistent_storage() {return 0}

=head3 can_store_empty_array()

=cut
sub can_store_empty_array() {return 0}

=head3 has_find()

=cut
sub has_find() {return 0}

=head3 has_clone()

=cut
sub has_clone() {return 0}

=head3 can_list_paths()

=cut
sub can_list_paths() {return 0}

=head3 has_keys_at()

=cut
sub has_keys_at() {return 0}


=head2 Data Access Methods

=head3 get()

get([path]) - fetch array of data stored at given path.
In array context returns the stored array.
In scalar context returns first element of the array.

Examples: 
my $first_val=$map->get([100,"July"]);
my @vals=$map->get([100,"July"]);
my @vals2=$map->get(\@path2);

=cut
sub get {
    my ($self, $pathptr)=@_;
    if (wantarray) {
	return @{$self->get_ref($pathptr)};
    } else {
	return $self->get_ref($pathptr)->[0];
    }
}

=head3 get_ref()

get_ref([path]) - the same as get([path]), but returns the reference to the array.
for efficiency reasons for some backends the referenced array may be not a copy of stored data, but the
data itself, so the got reference should be treated read-only.

=cut

sub get_ref_impl_with_get {
    my ($self, $pathptr)=@_;
    return [$self->get($pathptr)];
}

sub get_ref_impl_with_if {
    my ($self, $pathptr)=@_;
    my $ret = $self->get_ref_if($pathptr);
    return $ret? $ret : [];
}

*get_ref=*get_ref_impl_with_if;

=head3 get_ref_if()

get_ref_if([path]) - returns reference to array of data at given path
or undef if there was nothing stored. It tries to show the difference between
stored empty array and nothing stored. This difference is backend-specific:
Some backends feel the difference between stored empty array and stored nothing,
some does not.

=cut
sub get_ref_if {
    my ($self, $pathptr)=@_;
    my $ref=$self->get_ref($pathptr);
    if ($#{$ref}==0) {
	return undef;
    } else {
	return $ref;
    }
}

=head3 set()

set([path],[values]) - store array of data at given path.
It is designed to store scalar values.
note that for some backends (it depends on the backend implementation)
the perl-specific data such as object or subroutine references
can be stored incorrectly.

return value is backend-specific.

Examples: 
$map->set([100,"July"], [1]);
$map->set([100,"July"], \@data);
$map->set(\@path2, \@data2);

=cut
sub set {
    my ($self, $pathptr, $valueptr )= @_;
    Carp::croak "Array2ArrayMap: interface stub called for set";
}

=begin comment
=head3 put()

put([path],value) - store single value at given path.
return value is backend-specific.

It is method implemented directly in interface for convenience and compatibility reasons
(namely, for compatibility with Tie::ListKeyedHash).
Not recommended for use, as it can be a bit slower than native set() methods.

Examples: 
$map->put([100,"July"], 1);
$map->put(\@path, "Fog");

=end comment

=cut
sub put {
    my ($self, $pathptr, $value)= @_;
    return $self->set($pathptr,[$value]);
}

sub append {
    my ($self, $pathptr, $valueptr)= @_;
    my $data_ref=$self->get_ref($pathptr);
    push @$data_ref, @$valueptr;
    return $self->set($pathptr, $data_ref);
}

sub _subtract_arrays {
    my %hash=map {$_=>1} @{$_[1]};
    return grep {!$hash{$_}} @{$_[0]};
}

# sub _intersect_arrays {
#     my %hash=map {$_=>1} @{shift()};
#     return [grep {$hash{$_}} @{shift()}];
# }
# sub _unite_arrays {
#     my %tmp_hash=map {$_=>1} @{shift()}, @{shift()};
#     return [keys (%tmp_hash)];
# }

sub unique_append {
    # is a shortcut for remove+append
    my ($self, $pathptr, $valueptr)= @_;
    my $data_ref=$self->get_ref($pathptr);
    #    this implementation preserves the order of values
    my @newdata=_subtract_arrays($data_ref, $valueptr);
    push @newdata, @$valueptr;
    return $self->set($pathptr, \@newdata);
}

sub remove {
    my ($self, $pathptr,$valueptr)=@_;
    my $data_ref=$self->get_ref($pathptr);
    my @newdata=_subtract_arrays($data_ref, $valueptr);
    return $self->set($pathptr, \@newdata);
}

sub find {
    my ($self, $pathptr)=@_;
    Carp::croak "Array2ArrayMap: interface stub called for find";
}

#sub delete {
#    my ($self, $pathptr)=@_;
#    Carp::croak "Array2ArrayMap: interface stub called";
#}

# keys_at returns all keys at given path' level
sub keys_at {
    my ($self, $pathptr) = @_;
    Carp::croak "Array2ArrayMap: interface stub called for keys_at";
}

# returns paths that starts with [path]
sub paths_prefix {
    my ($self, $pathptr) = @_;
    my @pathlist = $self->subpaths_at($pathptr);
    map {unshift @$_, @$pathptr} @pathlist;
    return @pathlist;
}

# returns paths that ends with [path]
sub paths_suffix {
    Carp::croak "Array2ArrayMap: interface stub called for path_suffix";
}

# returns all paths
sub paths_all {
    Carp::croak "Array2ArrayMap: interface stub called for path_suffix";
}

# list subpaths that starts with $pathptr
sub subpaths_at {
    my ($self, $pathptr) = @_;
    my $shiftlen=@$pathptr;
    my @paths_at=$self->paths_prefix($pathptr);
    foreach (@paths_at) {
	for (my $i=$shiftlen;$i>0;$i--) {shift @$_}
    }
    return @paths_at;
}

# compatibility with Tie::ListKeyedHash; and tie HASH
# Not recommended for use
sub clearTOREMOVESUFFIXAFTERTEST {
    return &clear_prefix([]);
}

sub delete {
    return &clear_prefix;
}

sub clear_prefix {
    my ($self, $pathptr)=@_;
    foreach ($self->paths_prefix($pathptr)) {
	$self->delete($_);
    }
}

sub clear_suffix {
    my ($self, $pathptr)=@_;
    foreach ($self->paths_suffix($pathptr)) {
	$self->delete($_);
    }
}

# TODO: rename and implement!!!

## arguments path,object: deletes object on the branch after path
## hack - indicates code that needs special cache without find.
## ( notobject->attr->context1...->contextn)
## but rather attr->object[->join_context] (verify that 2 args!) by interface
sub all_clear {
    &delete;
}

#sub clone {
#    my $self=shift;
#    Carp::croak "Array2ArrayMap: interface stub called";
#}

########################## TIE::HASH ###############################

sub TIEHASH {
    bless {}, shift;
}

sub CLEAR {
    shift()->clear();
}

sub FETCH {
    my ($self, $keys) = @_;
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    $self->get_ref_if($keys);
}

sub STORE {
    my ($self, $keys, $value) = @_;
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    $self->rawput($keys,$value);
}

sub DELETE {
    my ($self, $keys) = @_;
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    $self->delete($keys);
}

sub EXISTS {
    my ($self, $keys) = @_;
    $keys = [$keys eq '' ? ('') : split /$;/, $keys, -1] unless ref $keys eq 'ARRAY';#/
    @$keys or croak "Empty multi-key\n";
    scalar $self->get_ref_if($keys);
}

sub FIRSTKEY {
    my ($self) = @_;
    Carp::croak "Array2ArrayMap: interface stub called";
}

sub NEXTKEY {
    my ($self) = @_;
    Carp::croak "Array2ArrayMap: interface stub called";
}

# stub for tie() 
sub SCALAR {1}

1;


__END__

=head1 NAME

Hash::MultiKey - hashes whose keys can be multiple

=head1 SYNOPSIS

  use Hash::MultiKey;

  # tie first
  tie %hmk, 'Hash::MultiKey';

  # store
  $hmk{['foo', 'bar', 'baz']} = 1;

  # fetch
  $v = $hmk{['foo', 'bar', 'baz']};

  # exists
  exists $hmk{['foo', 'bar', 'baz']}; # true

  # each
  while (($mk, $v) = each %hmk) {
      @keys = @$mk;
      # ...
  }

  # keys
  foreach $mk (keys %hmk) {
      @keys = @$mk;
      # ...
  }

  # values
  foreach $v (values %hmk) {
      $v =~ s/foo/bar/g; # alias, modifies value in %hmk
      # ...
  }

  # delete
  $rmed_value = delete $hmk{['foo', 'bar', 'baz']};

  # clear
  %hmk = ();

  # syntactic sugar, but see risks below
  $hmk{'foo', 'bar', 'baz', 'zoo'} = 2;

  # finally, untie
  untie %hmk;

=head1 DESCRIPTION

Hash::MultiKey provides hashes that accept arrayrefs of strings as keys.

Two multi-keys are regarded as being equal if their I<contents> are
equal, there is no need to use the same reference to refer to the same
hash entry:

    $hmk{['foo', 'bar', 'baz']} = 1;
    exists $hmk{['foo', 'bar', 'baz']}; # different arrayref, but true

A given hash can have multi-keys of different lengths:

    $hmk{['foo']}               = 1; # length 1
    $hmk{['foo', 'bar', 'baz']} = 3; # length 3, no problem

In addition, multi-keys cannot be empty:

    $hmk{[]} = 1; # ERROR

The next sections document how hash-related operations work in a
multi-key hash. Some parts have been copied from standard documentation,
since everything has standard semantics.

=head2 tie

Once you have tied a hash variable to Hash::MultiKey as in

    tie my (%hmk), 'Hash::MultiKey';

you've got a hash whose keys are arrayrefs of strings. Having that in
mind everything works as expected.

=head2 store

Assignment is this easy:

    $hmk{['foo', 'bar', 'baz']} = 1;

=head2 fetch

So is fetching:

    $v = $hmk{['foo', 'bar', 'baz']};

=head2 exists

Testing for existence works as usual:

    exists $hmk{['foo', 'bar', 'baz']}; # true

Only whole multi-keys as they were used in assigments have entries.
Sub-chains do not exist unless they were assigned some value.

For instance, C<['foo']> is a sub-chain of C<['foo', 'bar', 'baz']>, but
if it has no entry in %hmk so far

    exists $hmk{['foo']}; # false

=head2 each

As with everyday C<each()>, when called in list context returns a
2-element list consisting of the key and value for the next element of
the hash, so that you can iterate over it. When called in scalar
context, returns only the key for the next element in the hash.

Remember keys are arrayrefs of strings here:

    while (($mk, $v) = each %hmk) {
        @keys = @$mk;
        # ...
    }

The order in which entries are returned is guaranteed to be the same one
as either the C<keys()> or C<values()> function would produce on the
same (unmodified) hash.

When the hash is entirely read, a null array is returned in list context
(which when assigned produces a false (0) value), and C<undef> in scalar
context. The next call to C<each()> after that will start iterating
again.

There is a single iterator for each hash, shared by all C<each()>,
C<keys()>, and C<values()> function calls in the program.

Adding or deleting entries while we're iterating over the hash results
in undefined behaviour. Nevertheless, it is always safe to delete the
item most recently returned by C<each()>, which means that the following
code will work:

    while (($mk, $v) = each %hmk) {
        print "@$mk\n";
        delete $hmk{$mk}; # this is safe
    }

=head2 keys

Returns a list consisting of all the keys of the named hash. (In scalar
context, returns the number of keys.) The keys are returned in an
apparently random order. The actual random order is subject to change in
future versions of perl, but it is guaranteed to be the same order as
either the C<values()> or C<each()> function produces (given that the
hash has not been modified). As a side effect, it resets hash's
iterator.

Remember keys are arrayrefs of strings here:

    foreach $mk (keys %hmk) {
        @keys = @$mk;
        # ...
    }

There is a single iterator for each hash, shared by all C<each()>,
C<keys()>, and C<values()> function calls in the program.

The returned values are copies of the original keys in the hash, so
modifying them will not affect the original hash. Compare C<values()>.

=head2 values

Returns a list consisting of all the values of the named hash. (In a
scalar context, returns the number of values.) The values are returned
in an apparently random order. The actual random order is subject to
change in future versions of perl, but it is guaranteed to be the same
order as either the C<keys()> or C<each()> function would produce on the
same (unmodified) hash.

Note that the values are not copied, which means modifying them will
modify the contents of the hash:

   s/foo/bar/g foreach values %hmk;       # modifies %hmk's values
   s/foo/bar/g foreach @hash{keys %hash}; # same

As a side effect, calling C<values()> resets hash's internal iterator.

There is a single iterator for each hash, shared by all C<each()>,
C<keys()>, and C<values()> function calls in the program.


=head2 delete

Deletes the specified element(s) from the hash. Returns each element so
deleted or the undefined value if there was no such element.

The following (inefficiently) deletes all the values of %hmk:

    foreach $mk (keys %hmk) {
        delete $hmk{$mk};
    }

And so do this:

    delete @hmk{keys %hmk};

But both methods are slower than just assigning the empty list to %hmk:

    %hmk = (); # clear %hmk, the efficient way

=head2 untie

Untie the variable when you're done:

    untie %hmk;

=head1 SYNTACTIC SUGAR

Hash::MultiKey supports also this syntax:

    $hash{'see', '$;', 'in', 'perldoc', 'perlvar'} = 1;

If the key is a string instead of an arrayref the underlying code splits
it using C<$;> (see why in L<MOTIVATION>) and from then on the key is an
arrayref as any true multi-key. Thus, the assigment above is equivalent
to

    $hash{['see', '$;', 'in', 'perldoc', 'perlvar']} = 1;

once it has been processed.

You I<don't> need to split the string back while iterating with
C<each()> or C<keys()>, it already comes as an arrayref of strings.

Nevertheless take into account that this is B<slower>, and B<broken> if
any of the components contains C<$;>. It is supported just for
consistency's sake.


=head1 MOTIVATION

Perl comes already with some support for hashes with multi-keys. As you
surely know, if perl sees

    $hash{'foo', 'bar', 'baz'} = 1;

it joins C<('foo', 'bar', 'baz')> with C<$;> to obtain the actual key,
thus resulting in a string. Then you retrieve the components of the
multi-key like this:

    while (($k, $v) = each %hash) {
        @keys = $k eq '' ? ('') : split /$;/, $k, -1;
        # ...
    }

Since C<$;> is C<\034> by default, a non-printable character, this is
often enough.

Sometimes, however, that's not the most convenient way to work with
multi-keys. For instance, that magic join doesn't work with arrays:

    @array = ('foo', 'bar', 'baz');
    $hash{@array} = 1; # WARNING, @array evaluated in scalar context!

You could be dealing with binary data. Or you could be writing a public
module that uses user input in such a hash and don't want to rely on
input not coming with C<$;>, or don't want to document such an obscure,
gratuitous, and implementation dependent constraint.

In such cases, Hash::MultiKey can help.

=head1 AUTHORS

Xavier Noria (FXN), Benjamin Goldberg (GOLDBB).

=head1 THANKS

Iain Truskett (SPOON) kindly checked whether this module works in perl
5.005 and found out the use of "/" in C<pack()>, introduced in perl
5.006, prevents that.

Philip Monsen reported some tests of Hash::MultiKey 0.05 failed with
perl 5.8.4.

=head1 COPYRIGHT and LICENSE

Copyright (C) 2003, Xavier Noria E<lt>fxn@cpan.orgE<gt>. All rights
reserved. This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 SEE ALSO

L<perlvar>, L<perltie>

=cut



########################## Iner subs ###############################

sub perl_quote {
    my @args=@_;
    my @values;
    foreach my $value (@args) {
	if (! defined $value) {
	    push @values, 'undef';
	} elsif (ref $value eq 'CODE') {
	    # skip code, etc
	} elsif ($value eq '') {
	    push @values, "''";
	} elsif ($value=~/^\d+$/) {
	    push @values, $value;
	} else {
	    $value=~s/'/\\'/g; #';
	    push @values, "'$value'";
	}
    }
    return @values;
}

1;

=head1 BUGS

None yet :)

When submitting bug reports, be sure to include full details,
including the VERSION of the module, a test script and a test data
demonstrating the problem!

=head1 CREDITS

to Alexander Strelets, sav@imath.kiev.ua for his work on SDB backend.

=head1 WEBSITE

You can find information about Data::Array2ArrayMap at:

   http://html-tmpl-pro.sourceforge.net

=head1 PUBLIC GIT SERVER

Data::Array2ArrayMap now has a publicly accessible GIT repository at
http://git.altlinux.org/people/viy/ at 
(http://git.altlinux.org/people/viy/packages/?p=perl-Data-Array2ArrayMap.git;a=summary). 
Give it a try!

=head1 AUTHOR

I. Vlasenko, E<lt>viy@altlinux.orgE<gt>

=head1 LICENSE

  Data::Array2ArrayMap - perl interface for hierarchical data storage.
  Copyright (C) 2002-2007 I. Vlasenko, (viy@altlinux.org)

  This module is free software; you can redistribute it and/or modify it
  under the terms of either:

  a) the GNU General Public License as published by the Free Software
  Foundation; either version 2, or (at your option) any later version,
  
  or

  b) the "Artistic License" which comes with this module.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
  the GNU General Public License or the Artistic License for more details.

  You should have received a copy of the Artistic License with this
  module, in the file ARTISTIC.  If not, I'll be glad to provide one.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA

=cut
